package labsheet2;

import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.RunningJob;
import org.apache.hadoop.mapred.TextInputFormat;
import org.apache.hadoop.mapred.TextOutputFormat;
import org.apache.hadoop.mapreduce.JobID;

import labsheet1.WordCountMap;
import labsheet1.WordCountReduce;


/** 
 * Written by Dr. Richard McCreadie for the SICSA Summer School
 * on Big Data Information Visualisation.
 * 
 * Contact me if you want to use these for non-personal use.
 * richard.mccreadie@glasgow.ac.uk
 */
public class TopHashTags {

	public static void main(String[] args) throws Exception {
		JobConf conf = new JobConf(TopHashTags.class);
	    conf.setJobName("wordcount");
	    
	    conf.setOutputKeyClass(Text.class);
	    conf.setOutputValueClass(IntWritable.class);
	        
	    conf.setMapperClass(TopHashTagsMap.class);
	    conf.setReducerClass(TopHashTagsReduce.class);
	        
	    conf.setMapOutputKeyClass(Text.class);
	    conf.setMapOutputValueClass(IntWritable.class);
	    
	    conf.setInputFormat(TextInputFormat.class);
	    conf.setOutputFormat(TextOutputFormat.class);
	        
	    for (String file : args[0].split(",")) {
	    	FileInputFormat.addInputPath(conf, new Path(file));
	    }
	    FileOutputFormat.setOutputPath(conf, new Path(args[1]));
	        
	    JobID jobId = null;
	    boolean ranOK = true;
	    try{
	            RunningJob rj = JobClient.runJob(conf);
	            jobId = rj.getID();
	    } catch (Exception e) { 
	            ranOK = false;
	    }
	 }
	
}
