package labsheet3;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.compress.CompressionCodec;
import org.apache.hadoop.io.compress.GzipCodec;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.RecordWriter;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextOutputFormat;
import org.apache.hadoop.util.Progressable;
import org.apache.hadoop.util.ReflectionUtils;

/** 
 * Written by Dr. Richard McCreadie for the SICSA Summer School
 * on Big Data Information Visualisation.
 * 
 * Code is based on TextOutputFormat
 * 
 * Contact me if you want to use these for non-personal use.
 * richard.mccreadie@glasgow.ac.uk
 */
public class TRECOutputFormat<K, V> extends FileOutputFormat<K, V> {

	
	public RecordWriter<K, V> getRecordWriter(FileSystem ignored,
			JobConf job,
			String name,
			Progressable progress)
					throws IOException {
		boolean isCompressed = getCompressOutput(job);
		String keyValueSeparator = job.get("mapreduce.output.textoutputformat.separator", 
				"\t");
		if (!isCompressed) {
			Path file = FileOutputFormat.getTaskOutputPath(job, name);
			FileSystem fs = file.getFileSystem(job);
			FSDataOutputStream fileOut = fs.create(file, progress);
			return new TRECRecordWriter<K, V>(fileOut, keyValueSeparator);
		} else {
			Class<? extends CompressionCodec> codecClass =
					getOutputCompressorClass(job, GzipCodec.class);
			// create the named codec
			CompressionCodec codec = ReflectionUtils.newInstance(codecClass, job);
			// build the filename including the extension
			Path file = 
					FileOutputFormat.getTaskOutputPath(job, 
							name + codec.getDefaultExtension());
			FileSystem fs = file.getFileSystem(job);
			FSDataOutputStream fileOut = fs.create(file, progress);
			return new TRECRecordWriter<K, V>(new DataOutputStream
					(codec.createOutputStream(fileOut)),
					keyValueSeparator);
		}
	}
	
	protected static class TRECRecordWriter<K, V>
	implements RecordWriter<K, V> {
		private static final String utf8 = "UTF-8";
		private static final byte[] newline;
		static {
			try {
				newline = "\n".getBytes(utf8);
			} catch (UnsupportedEncodingException uee) {
				throw new IllegalArgumentException("can't find " + utf8 + " encoding");
			}
		}
		

		/**
		 * Write output in TREC format
		 * @param docno
		 * @param text
		 * @throws IOException
		 */
		private void writeTREC(Object docno, Object text) throws IOException {
			
			// --------------------------
			// SICSA Lab Sheet 3 - Part 2
			// --------------------------
			
		}
		

		protected DataOutputStream out;
		private final byte[] keyValueSeparator;

		public TRECRecordWriter(DataOutputStream out, String keyValueSeparator) {
			this.out = out;
			try {
				this.keyValueSeparator = keyValueSeparator.getBytes(utf8);
			} catch (UnsupportedEncodingException uee) {
				throw new IllegalArgumentException("can't find " + utf8 + " encoding");
			}
		}

		public TRECRecordWriter(DataOutputStream out) {
			this(out, "\t");
		}

		

		public synchronized void write(K key, V value)
				throws IOException {

			boolean nullKey = key == null || key instanceof NullWritable;
			boolean nullValue = value == null || value instanceof NullWritable;
			if (nullKey || nullValue) {
				return;
			} else {
				writeTREC(key, value);
			}
			
			
		}

		public synchronized void close(Reporter reporter) throws IOException {
			out.close();
		}
	}

	

}
